# Generated by Django 3.2.6 on 2021-08-30 15:03

import datetime

from django.db import migrations, models
import django.db.models.deletion


def slots_to_spans(hours):
    start = None
    end = None
    for hour in hours:
        if start is None:
            start = hour
        elif end != hour:
            yield start, end
            start = hour
        end = hour + 1
    if start:
        yield start, end


def migrate_data(apps, schema_editor):
    Subject = apps.get_model('subjects', 'Subject')
    AvailabilitySpan = apps.get_model('subjects', 'AvailabilitySpan')

    for subject in Subject.objects.all():
        for day, slots in [
            (0, subject.availability_monday),
            (1, subject.availability_tuesday),
            (2, subject.availability_wednesday),
            (3, subject.availability_thursday),
            (4, subject.availability_friday),
        ]:
            hours = slots.order_by('hour').values_list('hour', flat=True)
            for start, end in slots_to_spans(hours):
                AvailabilitySpan.objects.create(
                    subject=subject,
                    day=day,
                    start=datetime.time(start),
                    end=datetime.time(end),
                )


class Migration(migrations.Migration):

    dependencies = [
        ('subjects', '0024_subject_rm_note_fields'),
    ]

    operations = [
        migrations.CreateModel(
            name='AvailabilitySpan',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('start', models.TimeField(blank=True, null=True, verbose_name='start')),
                ('end', models.TimeField(blank=True, null=True, verbose_name='end')),
                ('day', models.SmallIntegerField(choices=[(0, 'Monday'), (1, 'Tuesday'), (2, 'Wednesday'), (3, 'Thursday'), (4, 'Friday'), (5, 'Saturday'), (6, 'Sunday')], verbose_name='day')),
                ('subject', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='subjects.subject')),
            ],
            options={
                'ordering': ['day', django.db.models.expressions.OrderBy(django.db.models.expressions.F('start'), nulls_first=True)],
            },
        ),
        migrations.RunPython(migrate_data),
        migrations.RemoveField(
            model_name='subject',
            name='availability_friday',
        ),
        migrations.RemoveField(
            model_name='subject',
            name='availability_monday',
        ),
        migrations.RemoveField(
            model_name='subject',
            name='availability_thursday',
        ),
        migrations.RemoveField(
            model_name='subject',
            name='availability_tuesday',
        ),
        migrations.RemoveField(
            model_name='subject',
            name='availability_wednesday',
        ),
        migrations.DeleteModel(
            name='TimeSlot',
        ),
    ]
